#include <gtk/gtk.h>

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <string.h>

#define NAME_POS 0
#define DATE_POS 40
#define PHONE_POS 52
#define ADDR_POS 74
#define ADVANCE_POS 113

#define NO_DEBUG_OUTPUT


/*
  Structure for holding matching records (those that will be later displayed).
  We hope that 100 records is pretty enough :)
*/
typedef struct RECORD
               {
	          char name[DATE_POS-NAME_POS+1];
		  char date[PHONE_POS-DATE_POS+1];
		  char phone[ADDR_POS-PHONE_POS+1];
		  char address[ADVANCE_POS-ADDR_POS+1];
		  char anniversary[70]; /* status line - "Somebody's anniversary is soon..." */
               } RECORDS;
	       
struct RECORD records[100];

int records_count=0;


int win_recode = FALSE;

/* 
 The following tables define russian alphabet in different codepages:
 
 win    - Windows-1251 codepage, used by Win 3.1x/95/98/NT;
 koi    - KOI8-R codepage, used by UNIX and most WWW pages;
*/

char win[64] = {    192,193,194,195,196,197,198,199,200,201,202,
                    203,204,205,206,207,208,209,210,211,212,213,214,
                    215,216,217,218,219,220, 221,222,223  ,
                    
                    224,225,226,227,228,229,230,231,232,233,234,
                    235,236,237,238,239,240,241,242,243,244,245,246,
                    247,248,249,250,251,252 ,253,254,255
               };
               
char koi[64] = {    225,226,247,231,228,229,246,250,233,234,235,
                    236,237,238,239,240,242,243,244,245,230,232,227,
                    254,251,253,255,249,248, 252,224,241  ,
                    
                    193,194,215,199,196,197,214,218,201,202,203,
                    204,205,206,207,208,210,211,212,213,198,200,195,
                    222,219,221,223,217,216, 220,192,209
               }; 

/*
Performs recoding between two russian codepages.
*/
char * rus_rus_xla(char * str, char* alf_in, char* alf_out)
{
   int i, j;
   char ch, ch1;
   
   for (i=0;i<strlen(str);i++)
   {
      ch = ch1 = str[i];
      
      for (j=0;j<64;j++)
      {
         if (alf_in[j]==ch)  
         { 
            ch1 = alf_out[j];
         }
      }
      str[i] = ch1;
   }
   return(str);
}

/*
Compares date from string with current date, and if the anniversary is
closer than days_allowed, updates age and returns TRUE
*/
int compare_date_field(char * str, int * diffdays, int * age)
{
   struct tm t,   /* today's time */ 
             t1;  /* anniversary's time */
   time_t tloc;
   int year;

   /* get today's time and copy it into both t and t1 */
   if (time(&tloc) == -1) return FALSE;
   t = *(gmtime(&tloc));
   t1 = t;

   if ( strlen(str) < (DATE_POS+8) )  return FALSE;
   
   /* now override t1's day and month with input data */
   t1.tm_mday = atoi(str+DATE_POS);
   t1.tm_mon = atoi(str+DATE_POS+3) - 1; /* tm_mon starts from zero... */
   year = atoi(str+DATE_POS+6);
  
#ifdef DEBUG_OUTPUT
   printf("Today: %d %d %d\n",  t.tm_mday, t.tm_mon+1, t.tm_year);
   printf("GetDate: %d %d %d\n",  t1.tm_mday, t1.tm_mon+1, t1.tm_year);
#endif
   /* how many days are left till anniversary */
   *diffdays = ( difftime(mktime(&t1),mktime(&t))/60/60/24 );
   
   /* the age of that lucky guy :) */
   *age = t.tm_year + 1900 - year;
   if (*diffdays < 0) (*age)++;
   
   return TRUE;
}

/*
Prints "Somebody's anniversary!" line
*/
void print_anniversary_line(int age, int diffdays)
{
   int last_digit = age - 10*(age/10);
   char digit_str[4],
        diffdays_str[12];
   
   switch (last_digit)
   {
     case 1:  sprintf(digit_str,"st"); break;
     case 2:  sprintf(digit_str,"nd"); break;
     case 3:  sprintf(digit_str,"rd"); break;
     default: sprintf(digit_str,"th");
   }
   
   switch (diffdays)
   {
     case 0:  sprintf(diffdays_str, "today"); break;
     case 1:  sprintf(diffdays_str,"tomorrow"); break;
     default: sprintf(diffdays_str,"in %d days", diffdays);
   }
   
   sprintf(records[records_count].anniversary,"Somebody's %d-%s anniversary %s!", age, digit_str, diffdays_str);
}


/*
Reads ADVANCE field from string
*/
int days_in_advance(char * str)
{
   return(atoi(str+ADVANCE_POS));
}


/*
Outputs the whole line information (Name, Address etc.)
*/
void print_entry_data(char * str)
{
   char name_str[DATE_POS-NAME_POS+1],
        date_str[PHONE_POS-DATE_POS+1],
        phone_str[ADDR_POS-PHONE_POS+1],
	addr_str[ADVANCE_POS-ADDR_POS+1];
	
   memcpy(name_str,str+NAME_POS,DATE_POS-NAME_POS);
   name_str[DATE_POS-NAME_POS] = 0;
   
   memcpy(date_str,str+DATE_POS,PHONE_POS-DATE_POS);
   date_str[PHONE_POS-DATE_POS] = 0;

   memcpy(phone_str,str+PHONE_POS,ADDR_POS-PHONE_POS);
   phone_str[ADDR_POS-PHONE_POS] = 0;

   memcpy(addr_str,str+ADDR_POS,ADVANCE_POS-ADDR_POS);
   addr_str[ADVANCE_POS-ADDR_POS] = 0;

   sprintf(records[records_count].name, "%s", win_recode ? rus_rus_xla(name_str, win, koi) : name_str);
   sprintf(records[records_count].date, "%s", win_recode ? rus_rus_xla(date_str, win, koi) : date_str);
   sprintf(records[records_count].phone, "%s", win_recode ? rus_rus_xla(phone_str, win, koi) : phone_str);
   sprintf(records[records_count].address, "%s", win_recode ? rus_rus_xla(addr_str, win, koi) : addr_str);
}


/*
Read input file line by line and call compare_date_field for deciding
wether this anniversary is close or not. If it is - datafill Records array.
*/
char read_input_file(FILE * in)
{
   char str[150];
   struct tm t;
   int err;
   int diffdays, age;
   char pause = FALSE;
   
   do
   {
      memset(str, 0, 150);
      fgets(str, 150, in);
#ifdef DEBUG_OUTPUT
      printf("%s", str);
#endif
      if (compare_date_field(str, &diffdays, &age))
      {
#ifdef DEBUG_OUTPUT
        printf("Diff: %d   Age: %d\n", diffdays, age);
#endif
        if ((diffdays >= 0) && (diffdays <= days_in_advance(str)))
        {
          print_anniversary_line(age, diffdays);
	  print_entry_data(str);
          records_count++;
	  pause = TRUE;
        }
      }
#ifdef DEBUG_OUTPUT
      else
        printf("Conversion failed :( %d\n", atoi(str));
#endif
        
   }
   while (!feof(in));
   
   return pause;
}

/*
   Construct GTK widget for our main window
*/
void show_window (void)
{
    int current_record;

    GtkWidget *window;
    GtkWidget *vbox, *hbox;
    GtkWidget *entry;
    GtkWidget *button;
    GtkWidget *check;
    GtkWidget *label;
    GtkWidget *table;
    GtkWidget *notebook;
    GtkWidget *frame;
    char buffer[32];

    /* create a new window */
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW (window), "Birthday Scanner");
    gtk_signal_connect(GTK_OBJECT (window), "delete_event",
                       (GtkSignalFunc) gtk_exit, NULL);

    table = gtk_table_new(3,3,FALSE);
    gtk_container_add (GTK_CONTAINER (window), table);
  
    /* Create a new notebook, place the position of the tabs */
    notebook = gtk_notebook_new ();
    gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
    gtk_table_attach_defaults(GTK_TABLE(table), notebook, 0,3,0,1);
    gtk_widget_show(notebook);
    
    /* Lets append a bunch of pages to the notebook */
    for (current_record=0; current_record < records_count; current_record++) 
    {
	sprintf(buffer, "Anniversary %d", current_record+1);
	
	frame = gtk_frame_new (NULL);
	gtk_container_set_border_width (GTK_CONTAINER (frame), 10);
	gtk_widget_set_usize (frame, 340, 180);
	gtk_widget_show (frame);
	

        vbox = gtk_vbox_new (FALSE, 0);
        gtk_container_add (GTK_CONTAINER (frame), vbox);
        gtk_widget_show (vbox);
	
	label = gtk_label_new (records[current_record].anniversary);
	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 5);
	gtk_widget_show (label);


	/* the Name field */
	hbox = gtk_hbox_new (FALSE, 5);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), 10);

	label = gtk_label_new ("Name:");
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	entry = gtk_entry_new_with_max_length (40);

	gtk_entry_set_text (GTK_ENTRY (entry), records[current_record].name);
	gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
	gtk_widget_show (entry);

	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
	gtk_widget_show (hbox);


	/* the Date field */
	hbox = gtk_hbox_new (FALSE, 5);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);

	label = gtk_label_new ("Date:");
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	entry = gtk_entry_new_with_max_length (12);

	gtk_entry_set_text (GTK_ENTRY (entry), records[current_record].date);
	gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
	gtk_widget_show (entry);

	/* the Phone field */
	label = gtk_label_new ("Phone:");
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	entry = gtk_entry_new_with_max_length (22);

	gtk_entry_set_text (GTK_ENTRY (entry), records[current_record].phone);
	gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
	gtk_widget_show (entry);

	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
	gtk_widget_show (hbox);

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (vbox), hbox);
	gtk_widget_show (hbox);


	/* the Address field */
	hbox = gtk_hbox_new (FALSE, 5);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);

	label = gtk_label_new ("Address:");
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	entry = gtk_entry_new_with_max_length (50);

	gtk_entry_set_text (GTK_ENTRY (entry), records[current_record].address);
	gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
	gtk_widget_show (entry);

	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
	gtk_widget_show (hbox);

                                  


	label = gtk_label_new (buffer);
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), frame, label);
    }


    /* Buttons */
    hbox = gtk_hbox_new (FALSE, 10);
                                  
    button = gtk_button_new_with_label ("Close");
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       GTK_SIGNAL_FUNC(gtk_exit),
			       GTK_OBJECT (window));
    gtk_table_attach_defaults(GTK_TABLE(table), button, 0,1,1,2);
    gtk_widget_show (button);

    button = gtk_button_new_with_label ("Previous");
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       (GtkSignalFunc) gtk_notebook_prev_page,
			       GTK_OBJECT (notebook));
    gtk_table_attach_defaults(GTK_TABLE(table), button, 1,2,1,2);
    gtk_widget_show (button);
    
    button = gtk_button_new_with_label ("Next");
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       (GtkSignalFunc) gtk_notebook_next_page,
			       GTK_OBJECT (notebook));
    gtk_table_attach_defaults(GTK_TABLE(table), button, 2,3,1,2);
    gtk_widget_show (button);

    gtk_widget_show (table);
    gtk_widget_show(window);
}

/*
Our main proc - I bet you knew it! :)
*/
int main (int argc, char *argv[])
{
   FILE * in;
   char str[150];
   struct tm t;
   int err;
   int diffdays, age;
   char pause_req = FALSE, pause = FALSE;
   
   if (argc < 2)
   {
      printf("\n=================================================\n");
      printf("Birthday Scanner, v0.2 for Linux, (c) by AGA 1999\n");
      printf("=================================================\n\n");
      printf("birthday <data_file> [win]\n\n"); 
      return(0);
   }
      
   if ((in = fopen(argv[1],"rt")) == NULL)
   {
      printf("Can't find source file - %s\n", argv[1]);
      return(0);
   }
   
   if (argc > 2)  win_recode = (strcmp(argv[2],"win") || strcmp(argv[2],"WIN"));
   
   if (read_input_file(in))
   {
      gtk_init (&argc, &argv);
      show_window();
      gtk_main();
   }
   
   fclose(in);
   
   return(0);
}
